#include "stdafx.h"
#include "..\..\DuiLib\UIlib.h"
#include "ChatLeftList.hpp"
#include <algorithm>

#include "ProfileWnd.h"
#include "main_frame.hpp"
#include "NeMessengerDlg.h"

namespace DuiLib
{

const int kFriendListItemNormalHeight = 23;
const int kFriendListItemSelectedHeight = 23;


CChatLeftList::CChatLeftList(CPaintManagerUI& paint_manager)
	: root_node_(NULL)
	, delay_deltaY_(0)
	, delay_number_(0)
	, delay_left_(0)
	, level_expand_image_(_T("<i list_icon_b.png>"))
	, level_collapse_image_(_T("<i list_icon_a.png>"))
	, level_text_start_pos_(0)
	, text_padding_(0, 7, 0, 0)
	, paint_manager_(paint_manager)
{
	SetItemShowHtml(true);

	root_node_ = new Node;
	root_node_->data().level_ = -1;
	root_node_->data().child_visible_ = true;
	root_node_->data().has_child_ = true;
	root_node_->data().list_elment_ = NULL;
	m_bEnableScroll = FALSE;
}

CChatLeftList::~CChatLeftList()
{
	if (root_node_)
		delete root_node_;

	root_node_ = NULL;
}

bool CChatLeftList::Add(CControlUI* pControl)
{
	if (!pControl)
		return false;

	if (_tcsicmp(pControl->GetClass(), _T("ListContainerElementUI")) != 0)
		return false;

	return CListUI::Add(pControl);
}

bool CChatLeftList::AddAt(CControlUI* pControl, int iIndex)
{
	if (!pControl)
		return false;

	if (_tcsicmp(pControl->GetClass(), _T("ListContainerElementUI")) != 0)
		return false;

	return CListUI::AddAt(pControl, iIndex);
}

bool CChatLeftList::Remove(CControlUI* pControl)
{
	if (!pControl)
		return false;

	if (_tcsicmp(pControl->GetClass(), _T("ListContainerElementUI")) != 0)
		return false;

	if (reinterpret_cast<Node*>(static_cast<CListContainerElementUI*>(pControl->GetInterface(_T("ListContainerElement")))->GetTag()) == NULL)
		return CListUI::Remove(pControl);
	else
		return RemoveNode(reinterpret_cast<Node*>(static_cast<CListContainerElementUI*>(pControl->GetInterface(_T("ListContainerElement")))->GetTag()));
}

bool CChatLeftList::RemoveAt(int iIndex)
{
	CControlUI* pControl = GetItemAt(iIndex);
	if (!pControl)
		return false;

	if (_tcsicmp(pControl->GetClass(), _T("ListContainerElementUI")) != 0)
		return false;

	if (reinterpret_cast<Node*>(static_cast<CListContainerElementUI*>(pControl->GetInterface(_T("ListContainerElement")))->GetTag()) == NULL)
		return CListUI::RemoveAt(iIndex);
	else
		return RemoveNode(reinterpret_cast<Node*>(static_cast<CListContainerElementUI*>(pControl->GetInterface(_T("ListContainerElement")))->GetTag()));
}

void CChatLeftList::RemoveAll()
{
	CListUI::RemoveAll();
	int nCount = root_node_->num_children();
	for (int i = nCount - 1; i >=0;  i--)
	{
		Node* child = root_node_->child(i);
		RemoveNode(child);
	}
	delete root_node_;

	root_node_ = new Node;
	root_node_->data().level_ = -1;
	root_node_->data().child_visible_ = true;
	root_node_->data().has_child_ = true;
	root_node_->data().list_elment_ = NULL;
}

void CChatLeftList::SetEnableScroll(BOOL bEnable)
{
	m_bEnableScroll = bEnable;
}

void CChatLeftList::DoEvent(TEventUI& event) 
{
	if (event.Type == UIEVENT_SCROLLWHEEL)
	{
        CPoint pt(0, 0);
		GetCursorPos(&pt);
		::ScreenToClient(GetManager()->GetPaintWindow(), &pt);
		CRect rcList = GetPos();
		if (!rcList.PtInRect(pt) || !m_bEnableScroll)
		{	
			return;
		}
	}
	CListUI::DoEvent(event);
}

int CALLBACK CChatLeftList::chatUsrSortFunc(void* pElem1,void* pElem2){
    int* status1 = (int*)pElem1 ;
    int* status2 = (int*)pElem2;
    return *status1 < *status2;  
}

 void CChatLeftList::UpdateName(CChatUser* pChatUser){
	 if (pChatUser == NULL )
	 {
		 return;
	 }

	 CString strID = pChatUser->GetID();
	 Node* node = NULL;
	 CString strUserID = _T("");
	 for (int i = 0; i < GetRoot()->num_children(); i++)
	 {
		 node = GetRoot()->child(i);
		 if(node != NULL)
		 {
			 strUserID = node->data().value;
			 if(strUserID == strID)
			 {
				 CLabelUI* nickname = static_cast<CLabelUI*>(paint_manager_.FindSubControlByName(node->data().list_elment_, kNickNameControlName));
				 if (nickname != NULL)
				 {					
					 nickname->SetText(pChatUser->GetName());					
				 }
				 break;
			 }
		 }
	 }
 }

void CChatLeftList::UpdateTooltip(CChatUser* pChatUser)
{
	
	if (pChatUser == NULL || pChatUser->GetEmail().GetLength() < 1)
	{
		return;
	}

	CString strID = pChatUser->GetID();
	Node* node = NULL;
	CString strUserID = _T("");
	for (int i = 0; i < GetRoot()->num_children(); i++)
	{
		node = GetRoot()->child(i);
		if(node != NULL)
		{
			strUserID = node->data().value;
			if(strUserID == strID)
			{
				CButtonUI* log_button = static_cast<CButtonUI*>(paint_manager_.FindSubControlByName(node->data().list_elment_, _T("logo")));
				CLabelUI* nickname = static_cast<CLabelUI*>(paint_manager_.FindSubControlByName(node->data().list_elment_, kNickNameControlName));
				if (log_button != NULL && nickname != NULL)
				{
					CString strBuf = _T("");
					strBuf.Format(LoadStringFromResource(IDS_STRING_RECENT_CHAT_TOOLTIP_3), pChatUser->GetName(), GetPresenceTip(pChatUser->GetStatus()), pChatUser->GetEmail());
					//log_button->SetToolTip(strBuf);
					nickname->SetToolTip(strBuf);

					const NE_MEMBER_DATA* data = g_pMessengerActor->GetEmployeeInfo(pChatUser->GetID());
					if (data)
					{
						if (ustOFFLINE == pChatUser->GetStatus() && GET_CONFIG()->EnableFunction(_T("UI_01")))
						{
							CString strVal = data->strWorkStatus.c_str();
							strVal.Replace(_T(":"),_T("\n"));
							log_button->SetToolTip(strVal);
						}
						else
						{
							log_button->SetToolTip(strBuf);
						}
					}
				}
				break;
			}
		}
	}
}

void CChatLeftList::UpdateNode(Node* node,  CString strImage)
{
	CButtonUI* log_button = static_cast<CButtonUI*>(paint_manager_.FindSubControlByName(node->data().list_elment_, _T("logo")));
	if (log_button != NULL)
	{
		log_button->SetBkImage(strImage);
		log_button->Invalidate();
	}
}

Node* CChatLeftList::GetRoot()
{
	return root_node_;
}

const TCHAR* const kLogoButtonControlName = _T("logo");
const TCHAR* const kLogoContainerControlName = _T("logo_container");
const TCHAR* const kNickNameControlName = _T("nickname");
const TCHAR* const kDescriptionControlName = _T("description");
const TCHAR* const kOperatorPannelControlName = _T("operation");

static bool OnLogoButtonEvent(void* event) {
	TEventUI *pEvent = (TEventUI*)event;
	static CString strOldImage = _T("");
    if( ((TEventUI*)event)->Type == UIEVENT_BUTTONDOWN ) {
        CControlUI* pButton = ((TEventUI*)event)->pSender;
        if( pButton != NULL ) {
            CListContainerElementUI* pListElement = (CListContainerElementUI*)(pButton->GetTag());
			CListUI *pThis = (CListUI *)pListElement->GetOwner();
			CPaintManagerUI* pm = pThis->GetManager();
			CRect rcItem = pButton->GetPos();
			Node* pNode = (Node*)pListElement->GetTag();
			if (pNode != NULL)
			{
				RECT rcClient = {0};
				GetWindowRect(pButton->GetManager()->GetPaintWindow(), &rcClient);
				RECT rcAround = {rcClient.left + rcItem.left, rcClient.top + rcItem.top, rcClient.left + rcItem.right, rcClient.top + rcItem.bottom};
				if (GET_CONFIG()->EnableFunction(_T("UI_03")))
				{
					GET_MESSENGER_DLG()->GetProfileDlgController()->CreateProfileDlg(pNode->data().value.GetData(), rcAround, pm->GetPaintWindow());
				}
			}

          //  if( pListElement != NULL ) pListElement->DoEvent(*(TEventUI*)event);
        }
    }
	else  if( ((TEventUI*)event)->Type == UIEVENT_MOUSEENTER )	
	{
		CControlUI* pButton = ((TEventUI*)event)->pSender;
		if( pButton != NULL ) 
		{
			CListContainerElementUI* pListElement = (CListContainerElementUI*)(pButton->GetTag());
			if( pListElement != NULL ) 
			{
				CChatLeftList *pThis = (CChatLeftList *)pListElement->GetOwner();
				CButtonUI* logo_hot_button = static_cast<CButtonUI*>(pThis->paint_manager_.FindSubControlByName(pListElement, _T("logo")));
				if (logo_hot_button!=NULL )
				{						
					Node* pNode = (Node*)pListElement->GetTag();
					CString strImage = GetAppDUIImgDir() +_T("\\btn_profile_icon.png");
					strOldImage = logo_hot_button->GetBkImage();
					logo_hot_button->SetBkImage(strImage);
				}
				pListElement->DoEvent(*pEvent);
			}
		}
	}
	else if(((TEventUI*)event)->Type== UIEVENT_MOUSELEAVE )
	{
		CControlUI* pButton =((TEventUI*)event)->pSender;
		if( pButton != NULL ) 
		{
			CListContainerElementUI* pListElement = (CListContainerElementUI*)(pButton->GetTag());
			if( pListElement != NULL ) 
			{
				CChatLeftList *pThis = (CChatLeftList *)pListElement->GetOwner();
				CButtonUI* log_hot_button = static_cast<CButtonUI*>(pThis->paint_manager_.FindSubControlByName(pListElement, _T("logo")));
				if (log_hot_button!=NULL)
				{				
					
					log_hot_button->SetBkImage(strOldImage);
				}
				pListElement->DoEvent(*pEvent);
			}
		}
	}
    return true;
}

Node* CChatLeftList::AddNode(const ChatListItemInfo& item, int nCompareData, Node* parent)
{
	if (!parent)
		parent = root_node_;

    CListContainerElementUI* pListElement = NULL;
    if( !m_dlgBuilder.GetMarkup()->IsValid() ) {
        pListElement = static_cast<CListContainerElementUI*>(m_dlgBuilder.Create(_T("chat_user_list_item.xml"), (UINT)0, NULL, &paint_manager_));
    }
    else {
        pListElement = static_cast<CListContainerElementUI*>(m_dlgBuilder.Create((UINT)0, &paint_manager_));
    }
    if (pListElement == NULL)
        return NULL;

	//pListElement->SetCompareData(nCompareData);

	Node* node = new Node;

	node->SetComapreData(nCompareData);

	node->data().level_ = parent->data().level_ + 1;
	
	node->data().has_child_ = false;
	node->data().child_visible_ = (node->data().level_ == 0);
	node->data().child_visible_ = false;

	node->data().text_ = item.nick_name;
	node->data().value = item.id;
	node->data().status = item.status;
	node->data().list_elment_ = pListElement;

	if (!parent->data().child_visible_)
		pListElement->SetVisible(false);

	if (parent != root_node_ && !parent->data().list_elment_->IsVisible())
		pListElement->SetVisible(false);

	CDuiRect rcPadding = text_padding_;
	for (int i = 0; i < node->data().level_; ++i)
	{
		rcPadding.left += level_text_start_pos_;		
	}
	pListElement->SetPadding(rcPadding);

	CLabelUI* log_button = static_cast<CLabelUI*>(paint_manager_.FindSubControlByName(pListElement, kLogoButtonControlName));
	if (log_button != NULL)
	{
		if (!item.logo.IsEmpty())
		{
			log_button->SetBkImage(item.logo);
		}
		else
		{
			CContainerUI* logo_container = static_cast<CContainerUI*>(paint_manager_.FindSubControlByName(pListElement, kLogoContainerControlName));
			if (logo_container != NULL)
				logo_container->SetVisible(false);
		}
        log_button->SetTag((UINT_PTR)pListElement);
        log_button->OnEvent += MakeDelegate(&OnLogoButtonEvent);
	}

	CLabelUI* nick_name = static_cast<CLabelUI*>(paint_manager_.FindSubControlByName(pListElement, kNickNameControlName));
	if (nick_name != NULL)
	{
		nick_name->SetTextColor(0xFF5F6164);
		nick_name->SetText(item.nick_name);
	}

	pListElement->SetFixedHeight(kFriendListItemNormalHeight);
	pListElement->SetTag((UINT_PTR)node);
	int index = 0;
	int innerIndex = 0;
	int iSumChildCount = 0;
	if (parent->has_children())
	{
		Node* prev = parent->get_last_child();
		index = prev->data().list_elment_->GetIndex() + 1;
		for (int i = 0; i < parent->num_children(); i++)
		{
			Node* pNode = parent->child(i);
			if(pNode->GetCompareData() > nCompareData)
			{
				index = pNode->data().list_elment_->GetIndex();
				break;
			}
			else
			{
				iSumChildCount += pNode->num_children();
			}
		}

		if (parent == root_node_)
		{
			innerIndex = index - iSumChildCount;
		}
		else
		{
			innerIndex = index - parent->data().list_elment_->GetIndex() - 1;
		}
		
	}
	else 
	{
		if (parent == root_node_)
			index = 0;
		else
		{
			index = parent->data().list_elment_->GetIndex() + 1;
		}
		innerIndex = 0;
	}
	if (!CListUI::AddAt(pListElement, index))
	{
		delete pListElement;
		delete node;
		node = NULL;
	}

	parent->insert_child(node, innerIndex);
	return node;
}


void CChatLeftList::ResortChatList(){

	if (root_node_->num_children() < 2)	
		return;

    CListUI::RemoveAll();
	
    for (int i=0; i< root_node_->num_children();i++)
    {
         Node *pNode =  root_node_->child(i);
		 CListContainerElementUI* pListElement = pNode->data().list_elment_;
		 if (!CListUI::AddAt(pListElement, i))
		 {
			 delete pListElement;
			 //delete node;
			 root_node_->remove_child(pNode);
		 }
    }
}

bool CChatLeftList::RemoveNode(Node* node)
{
	if (!node || node == root_node_) return false;

	int nCount = node->num_children();
	for (int i = nCount - 1; i >=0;  i--)
	{
		Node* child = node->child(i);
		RemoveNode(child);
	}

	CListUI::Remove(node->data().list_elment_);
	node->parent()->remove_child(node);
	delete node;

	return true;
}

bool CChatLeftList::RemoveNodeByUserID(LPCTSTR strUserID)
{
	for (int i = 0; i < root_node_->num_children(); ++i)
	{
		Node* child = root_node_->child(i);
		if (child->data().value == strUserID)
		{
			return RemoveNode(child);
		}
	}
	return false;
}

Node* CChatLeftList::FindNodeByUserID(LPCTSTR strUserID){
	for (int i = 0; i < root_node_->num_children(); ++i)
	{
		Node* child = root_node_->child(i);
		if (child->data().value == strUserID)
		{
			return child;
		}
	}
	return NULL;
}

void CChatLeftList::SetChildVisible(Node* node, bool visible)
{
	if (!node || node == root_node_)
		return;

	if (node->data().child_visible_ == visible)
		return;

	node->data().child_visible_ = visible;

	TCHAR szBuf[MAX_PATH] = {0};
	CDuiString html_text;
	if (node->data().has_child_)
	{
		if (node->data().child_visible_)
			html_text += level_expand_image_;
		else
			html_text += level_collapse_image_;

#if defined(UNDER_WINCE)
		_stprintf(szBuf, _T("<x %d>"), 10);
#else
		_stprintf_s(szBuf, MAX_PATH - 1, _T("<x %d>"), 10);
#endif
		html_text += szBuf;

		html_text += node->data().text_;

		CLabelUI* nick_name = static_cast<CLabelUI*>(paint_manager_.FindSubControlByName(node->data().list_elment_, kNickNameControlName));
		if (nick_name != NULL)
		{
			nick_name->SetShowHtml(true);
			nick_name->SetText(html_text);
		}
	}

	if (!node->data().list_elment_->IsVisible())
		return;

	if (!node->has_children())
		return;

	Node* begin = node->child(0);
	Node* end = node->get_last_child();
	for (int i = begin->data().list_elment_->GetIndex(); i <= end->data().list_elment_->GetIndex(); ++i)
	{
		CControlUI* control = GetItemAt(i);
		if (_tcsicmp(control->GetClass(), _T("ListContainerElementUI")) == 0)
		{
			if (visible) 
			{
				Node* local_parent = ((Node*)control->GetTag())->parent();
				if (local_parent->data().child_visible_ && local_parent->data().list_elment_->IsVisible())
				{
					control->SetVisible(true);
				}
			}
			else
			{
				control->SetVisible(false);
			}
		}
	}
}

bool CChatLeftList::CanExpand(Node* node) const
{
	if (!node || node == root_node_)
		return false;

	return node->data().has_child_;
}

list<Node*> CChatLeftList::GetSelectedItems()
{
	list<Node*> listRet;
	list<int> listSelected = CListUI::GetSelectedItems();
	for(list<int>::iterator it = listSelected.begin();
		it != listSelected.end(); it++)
	{
		CControlUI* pControl = GetItemAt(*it);
		//Node* node = NULL;
		if( pControl != NULL) 
		{
			Node* node = (Node*)pControl->GetTag();
			if(node != NULL)
			{
				listRet.push_back(node);
			}
		}
	}
	
	return listRet;
}

void CChatLeftList::SetSelectViewModal( CControlUI* pControl, bool bSelect)
{
	int nHeight = bSelect ? kFriendListItemSelectedHeight : kFriendListItemNormalHeight;
	CListContainerElementUI* pFriendListItem = static_cast<CListContainerElementUI*>(pControl);
	Node* node = (Node*)pControl->GetTag();
	if ((pFriendListItem != NULL) && (node != NULL) && !node->folder())
	{
		pFriendListItem->SetFixedHeight(nHeight);
		CContainerUI* pOperatorPannel = static_cast<CContainerUI*>(m_pManager->FindSubControlByName(pFriendListItem, kOperatorPannelControlName));
		if (pOperatorPannel != NULL)
		{
			pOperatorPannel->SetVisible(bSelect);
		}
	}
}

void CChatLeftList::UpdateStatus( CChatUser* pChatUser, GTUserStatusType stStatus )
{
	UpdateTooltip(pChatUser);
	Node* node = NULL;
	for (int i = 0; i < GetRoot()->num_children(); i++)
	{
		node = GetRoot()->child(i);
		if(node)
		{
			if(pChatUser->GetID() == node->data().value)
			{
				GTUserStatusType UserStatus = stStatus;
				CString strImg;
				if(GET_MESSENGER_DLG()->GetMainFrame())
					strImg = GET_MESSENGER_DLG()->GetMainFrame()->GetImageFromStatus(UserStatus);

				UpdateNode(node, strImg);

				break;
			}
		}
	}
}

} // namespace DuiLib